<?php
//
// $Id: class.ftp.inc,v1.0 2004-11-23 5:10p EST Onion Exp $
//
/*
 * FTP Class
 */
class FTP {

 /*
  * Public Properties
  */
 public $host = '';
 public $port = 0;
 public $user = '';
 public $pass = '';
 public $welcome_msg = '';
 public $last_error_msg = '';

 /*
  * Private Properties
  */
 private $ftp_sock;
 private $data_sock;
 private $response_log = array ();

 /*
  * Magic Methods
  */
  /*
   * Constructor
   */
 public function __construct ( $host, $port = 21, $user = 'anonymous', $pass = 'webmaster@goatse.cx' ) {

  $this->host = $host;
  $this->port = (int) $port;
  $this->user = $user;
  $this->pass = $pass;

  $this->ftp_sock = @fsockopen ($this->host, $this->port, $errno, $errstr);
  if ($this->ftp_sock === false) {
   $this->trigger_error ($errno.': '.$errstr);
  } else {
   
   $this->welcome_msg = $this->log_response (1);
   $this->write ('USER '.$user);
   $this->write ('PASS '.$pass);

  }
 
 }

  /*
   * Destructor
   */
 public function __destruct ( ) {
  if ($this->ftp_sock !== false) {
   $this->write ('QUIT');
   fclose ($this->ftp_sock);
  }
 }

  /*
   * toString Method
   */
 public function __toString ( ) {
  return $this->log_response (1);
 }

 /*
  * Static Methods
  */
  /*
   * bool is_error ( int number )
   */
 public static function is_error ( $number ) {
  return ($number >= 500);
 }

  /*
   * bool is_ok ( int number )
   */
 public static function is_ok ( $number ) {
  return ($number < 500);
 }

 /*
  * Public Methods
  */
  /*
   * void trigger_error ( string error )
   */
 public function trigger_error ( $error ) {
  $this->last_error_msg = $error;
 }

  /*
   * bool allo ( int size ) 
   */
 public function allo ( $size ) {
  $this->write ('ALLO '.$size);
  return self::is_ok ($this->get_last_response_num ());
 }

  /*
   * bool cwd ( string dir )
   */
 public function cwd ( $dir ) {
  $this->write ('CWD '.$dir);
  return self::is_ok ($this->get_last_response_num ());
 }

  /*
   * bool cdup ( void )
   */
 public function cdup ( ) {
  $this->write ('CDUP');
  return self::is_ok ($this->get_last_response_num ());
 }

  /*
   * bool dele ( string path )
   */
 public function dele ( $path ) {
  $this->write ('DELE '.$path);
  return self::is_ok ($this->get_last_response_num ());
 }

  /*
   * string ls ( [string dir] )
   */
 public function ls ( $dir = null ) {
  $this->write ('PASV');
  preg_match ('#\((\d{1,3}),(\d{1,3}),(\d{1,3}),(\d{1,3}),(\d{1,3}),(\d{1,3})\)#', $this->get_last_response_str (), $p);
  $this->create_data_sock ("{$p[1]}.{$p[2]}.{$p[3]}.{$p[4]}", ((int) $p[5] << 8) + $p[6]);
  $this->write ('TYPE A');
  $this->write ('LIST'.($dir === null ? '' : ' '.$dir));
  return $this->read_data_sock ();
 }

  /*
   * bool mkd ( string dirname )
   */
 public function mkd ( $dirname ) {
  $this->write ('MKD '.$dirname);
 }

  /*
   * string nlst ( [string dir] )
   */
 public function nls ( $dir = null ) {
  $this->write ('PASV');
  preg_match ('#\((\d{1,3}),(\d{1,3}),(\d{1,3}),(\d{1,3}),(\d{1,3}),(\d{1,3})\)#', $this->get_last_response_str (), $p);
  $this->create_data_sock ("{$p[1]}.{$p[2]}.{$p[3]}.{$p[4]}", ((int) $p[5] << 8) + $p[6]);
  $this->write ('TYPE A');
  $this->write ('NLST'.($dir === null ? '' : ' '.$dir));
  return $this->read_data_sock ();
 }

  /*
   * string pwd ( void )
   */
 public function pwd ( ) {
  $this->write ('PWD');
  preg_match ('#"(.*?)"#', $this->get_last_response_str (), $pwd_match);
  return $pwd_match[1];
 }

  /*
   * bool rename ( string from, string to )
   */
 public function rename ( $from, $to ) {
  $this->write ('RNFR '.$from);
  $this->write ('RNTO '.$to);
  return self::is_ok ($this->get_last_response_num ());
 }

  /*
   * string retr ( string filename )
   */
 public function retr ( $filename ) {
  $this->write ('PASV');
  preg_match ('#\((\d{1,3}),(\d{1,3}),(\d{1,3}),(\d{1,3}),(\d{1,3}),(\d{1,3})\)#', $this->get_last_response_str (), $p);
  $this->create_data_sock ("{$p[1]}.{$p[2]}.{$p[3]}.{$p[4]}", ((int) $p[5] << 8) + $p[6]);
  $this->write ('RETR '.$filename);
  return self::is_ok ($this->get_last_response_num ()) ? $this->read_data_sock () : false;
 }
  
  /*
   * bool rmd ( string dir )
   */
 public function rmd ( $dir ) {
  $this->write ('RMD '.$dir);
  return self::is_ok ($this->get_last_response_num ());
 }

  /*
   * bool site ( string cmd ) 
   */
 public function site ( $cmd ) {
  $this->write ('SITE '.$cmd);
  return self::is_ok ($this->get_last_response_num ());
 }

  /*
   * void stor ( string local_filename [, string remote_filename ] )
   */
 public function stor ( $local_filename, $remote_filename = null ) {
  $remote_filename === null ? $remote_filename = $local_filename : '';
  $this->write ('PASV');
  preg_match ('#\((\d{1,3}),(\d{1,3}),(\d{1,3}),(\d{1,3}),(\d{1,3}),(\d{1,3})\)#', $this->get_last_response_str (), $p);
  $this->create_data_sock ("{$p[1]}.{$p[2]}.{$p[3]}.{$p[4]}", ((int) $p[5] << 8) + $p[6]);
  $this->write ('STOR '.$remote_filename);
  if (self::is_ok ($this->get_last_response_num ())) {
   $this->write_data_sock (file_get_contents ($local_filename));
  }
  return self::is_ok ($this->get_last_response_num ());
 }

  /*
   * string syst ( void )
   */
 public function syst ( ) {
  $this->write ('SYST');
  return $this->get_last_response_str ();
 }

  /*
   * string get_last_response ( void )
   */
 public function get_last_response ( ) {
  return end ($this->response_log);
 }

  /*
   * int get_last_response_num ( void )
   */
 public function get_last_response_num ( ) {
  return preg_match ('#^(\d{1,3}).*$#', $this->get_last_response (), $matches) ? (int) $matches[1] : 0;
 }

  /*
   * string get_last_response_str ( void )
   */
 public function get_last_response_str ( ) {
  return preg_match ('#^\d{1,3}\s*(.*)$#', $this->get_last_response (), $matches) ? $matches[1] : '';
 }
 /*
  * Private Methods
  */
  /*
   * void write ( string string )
   */
 private function write ( $string ) {
  fwrite ($this->ftp_sock, "$string\r\n");
  $this->log_response ();
 }

  /*
   * string|void log_response ( [bool return] )
   */
 private function log_response ( $return = false ) {
  while ($line = fgets ($this->ftp_sock)) {
   $this->response_log[] = $line;
  }
  if ($return) {
   return implode ('', $this->response_log);
  }
 }

  /*
   * void create_data_sock ( string addr, int port )
   */
 private function create_data_sock ( $addr, $port ) {
  $this->data_sock = fsockopen ($addr, $port, $errno, $errstr);
  if ($this->data_sock === false) {
   $this->trigger_error ($errno.': '.$errstr);
  }
 }

  /*
   * string read_data_sock ( void )
   */
 private function read_data_sock ( ) {
  $data = '';
  while ($line = fgets ($this->data_sock, 8192)) {
   $data .= $line;
  }
  fclose ($this->data_sock);
  return $data;
 }

  /*
   * void write_data_sock ( string buffer )
   */
 private function write_data_sock ( $buffer ) {
  fwrite ($this->data_sock, $buffer);
  fclose ($this->data_sock);
 }

}
